import base64
from unittest.mock import AsyncMock, patch, MagicMock
import os
import pytest
import sys
import json
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', 'src', 'lmnt')))
from api import Speech, StreamingSynthesisConnection, _SYNTHESIZE_STREAMING_ENDPOINT  # noqa


MOCK_AUDIO = 'UklGRiQAAABXQVZFZm10IBAAAAABAAEAIlYAAESsAAACABAAZGF0YQAAAAA='
MOCK_RESPONSE_OBJ = {'audio': MOCK_AUDIO, 'durations': [{'text': 'Hello,', 'start': 0.0,
                                                         'duration': 0.5}, {'text': 'world!', 'start': 0.5, 'duration': 0.5}], 'seed': 'random_seed'}


@pytest.fixture
async def api():
  with patch('aiohttp.ClientSession', new=MagicMock()) as MockClientSession:
    key = 'test_key'
    api = Speech(key)
    api._lazy_init()
    MockClientSession.assert_called_once()
    yield api


@pytest.mark.asyncio
async def test_synthesize(api):
  text = 'Hello, world!'
  voice = 'Voice1'
  mock_response = {'audio': MOCK_AUDIO, 'durations': [], 'seed': 'random_seed'}
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  synthesis_result = await api.synthesize(text, voice)
  assert synthesis_result == {'audio': base64.b64decode(mock_response['audio'])}


@pytest.mark.asyncio
async def test_synthesize_return_durations(api):
  text = 'Hello, world!'
  voice = 'Voice1'
  mock_response = MOCK_RESPONSE_OBJ
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  synthesis_result = await api.synthesize(text, voice, return_durations=True)
  assert synthesis_result == {'audio': base64.b64decode(mock_response['audio']), 'durations': mock_response['durations']}


@pytest.mark.asyncio
async def test_synthesize_return_seed(api):
  text = 'Hello, world!'
  voice = 'Voice1'
  mock_response = {'audio': MOCK_AUDIO, 'durations': [], 'seed': 'random_seed'}
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  synthesis_result = await api.synthesize(text, voice, return_seed=True)
  assert synthesis_result == {'audio': base64.b64decode(mock_response['audio']), 'seed': mock_response['seed']}


@pytest.mark.asyncio
async def test_synthesize_return_durations_and_seed(api):
  text = 'Hello, world!'
  voice = 'Voice1'
  mock_response = MOCK_RESPONSE_OBJ
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  synthesis_result = await api.synthesize(text, voice, return_durations=True, return_seed=True)
  assert synthesis_result == {'audio': base64.b64decode(mock_response['audio']), 'durations': mock_response['durations'], 'seed': mock_response['seed']}


@pytest.mark.asyncio
async def test_synthesize__non_en_language(api):
  text = 'Hello, world!'
  voice = 'Voice1'
  language = 'pt'
  mock_response = {'audio': MOCK_AUDIO, 'durations': [], 'seed': 'random_seed'}
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  synthesis_result = await api.synthesize(text, voice, language=language)
  assert synthesis_result == {'audio': base64.b64decode(mock_response['audio'])}


@pytest.mark.asyncio
async def test_synthesize_no_text(api):
  with pytest.raises(AssertionError):
    await api.synthesize(None, 'Voice1')


@pytest.mark.asyncio
async def test_synthesize_no_voice(api):
  with pytest.raises(AssertionError):
    await api.synthesize('Hello, world!', None)


@pytest.mark.asyncio
async def test_deprecated_durations(api):
  text = 'Hello, world!'
  voice = 'Voice1'
  mock_response = MOCK_RESPONSE_OBJ
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  synthesis_result = await api.synthesize(text, voice, durations=True)
  assert synthesis_result == {'audio': base64.b64decode(mock_response['audio']), 'durations': mock_response['durations']}


@pytest.mark.asyncio
async def test_deprecated_durations_false(api):
  text = 'Hello, world!'
  voice = 'Voice1'
  mock_response = MOCK_RESPONSE_OBJ
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  synthesis_result = await api.synthesize(text, voice, durations=False, return_durations=True)
  assert synthesis_result == {'audio': base64.b64decode(mock_response['audio']), 'durations': mock_response['durations']}


@pytest.mark.asyncio
async def test_synthesize_streaming(api):
  voice = 'Voice1'
  speed = 1.5
  expressive = 0.8
  return_extras = True
  language = 'pt'

  mock_ws = AsyncMock()
  api._session = AsyncMock()
  api._session.ws_connect.return_value = mock_ws

  connection = await api.synthesize_streaming(voice, return_extras=return_extras, speed=speed, expressive=expressive, language=language)

  assert isinstance(connection, StreamingSynthesisConnection)
  api._session.ws_connect.assert_called_once_with(f'{api._base_url}{_SYNTHESIZE_STREAMING_ENDPOINT}')
  mock_ws.send_str.assert_called_once_with(json.dumps({
      'X-API-Key': api._api_key,
      'voice': voice,
      'speed': speed,
      'expressive': expressive,
      'send_extras': return_extras,
      'language': language
  }))


@pytest.mark.asyncio
async def test_synthesize_streaming_defaults(api):
  voice = 'Voice1'

  mock_ws = AsyncMock()
  api._session = AsyncMock()
  api._session.ws_connect.return_value = mock_ws

  connection = await api.synthesize_streaming(voice)

  assert isinstance(connection, StreamingSynthesisConnection)
  api._session.ws_connect.assert_called_once_with(f'{api._base_url}{_SYNTHESIZE_STREAMING_ENDPOINT}')
  mock_ws.send_str.assert_called_once_with(json.dumps({
      'X-API-Key': api._api_key,
      'voice': voice,
      'send_extras': False
  }))



# Copyright 2023 LMNT, Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================

from aiohttp import WSMsgType
from typing import List, Optional

import aiohttp
import base64
import json
import os


_BASE_URL = 'https://api.lmnt.com'
_SYNTHESIZE_STREAMING_ENDPOINT = '/v1/ai/speech/stream'
_LIST_VOICES_ENDPOINT = '/v1/ai/voice/list'
_VOICE_ENDPOINT = '/v1/ai/voice/{id}'
_CREATE_VOICE_ENDPOINT = '/v1/ai/voice'
_SPEECH_ENDPOINT = '/v1/ai/speech'
_ACCOUNT_ENDPOINT = '/v1/account'


class SpeechError(Exception):
  def __init__(self, status, error, caller):
    self.message = ''
    if caller:
      self.message = f'[{caller}]: '
    self.status = status
    if 'error' in error:
      self.message += error['error']
    elif 'message' in error:
      self.message += error['message']
    else:
      self.message += 'Unknown error; see status code for hints on what went wrong.'

  def __str__(self):
    return f'SpeechError [status={self.status}] {self.message}'


class StreamError(Exception):
  def __init__(self, message):
    self.message = message

  def __str__(self):
    return f'StreamError: {self.message}'


class _StreamingSynthesisIterator:
  def __init__(self, original_iterator, return_extras: bool):
    self.original_iterator = original_iterator
    self.return_extras = return_extras

  async def __anext__(self):
    msg1 = await self.original_iterator.__anext__()
    if msg1.type == WSMsgType.CLOSE:
      return  # Equivalent to raising a StopAsyncIteration exception, will cleanly stop async for loop
    data = {}

    if self.return_extras:
      msg1_json = self._parse_and_check_errors(msg1, require_error=False)
      msg2 = await self.original_iterator.__anext__()
      if msg2.type != WSMsgType.BINARY:
        self._parse_and_check_errors(msg2)
      data = {'audio': msg2.data, 'durations': msg1_json['durations']}
      if 'warning' in msg1_json:
        data['warning'] = msg1_json['warning']
      if 'buffer_empty' in msg1_json:
        data['buffer_empty'] = msg1_json['buffer_empty']
    else:
      if msg1.type != WSMsgType.BINARY:
        self._parse_and_check_errors(msg1)
      data = {'audio': msg1.data}

    return data

  def _parse_and_check_errors(self, msg, require_error=True):
    if msg.type != WSMsgType.TEXT:
      raise StreamError('Unexpected message type received from server.')
    msg_json = json.loads(msg.data)
    if 'error' in msg_json:
      raise StreamError(msg_json['error'])
    if require_error:
      raise StreamError('Unexpected message type received from server.')
    return msg_json


class StreamingSynthesisConnection:
  def __init__(self, socket, return_extras: bool):
    self.socket = socket
    self.return_extras = return_extras

  def __aiter__(self):
    """
    Returns a streaming iterator that yields an object containing binary audio data (and optionally other data) as it is received from the server.
    """
    return _StreamingSynthesisIterator(self.socket.__aiter__(), self.return_extras)

  async def __anext__(self):
    return self.socket.__anext__()

  async def append_text(self, text: str):
    msg = {
        'text': text
    }
    await self.socket.send_str(json.dumps(msg))

  async def flush(self):
    await self.socket.send_str('{"flush": true}')

  async def finish(self):
    await self.socket.send_str('{"eof": true}')


class Speech:
  def __init__(self, api_key: Optional[str] = None, **kwargs):
    self._session = None
    self._api_key = api_key or os.environ.get('LMNT_API_KEY')
    self._base_url = kwargs.get('base_url', _BASE_URL)
    self._connector = kwargs.get('connector', None)
    if not self._api_key:
      raise ValueError('Please set the `LMNT_API_KEY` environment variable or pass it in to the Speech constructor.')

  async def __aenter__(self):
    self._lazy_init()
    return self

  async def __aexit__(self, exc_type, exc, tb):
    await self.close()

  async def close(self):
    if self._session is not None:
      await self._session.close()
    self._session = None

  async def list_voices(self, starred: bool = False, owner: str = 'all'):
    """
    Returns a list of voices available to you.

    Optional parameters:
    - `starred`: Show starred voices only. Defaults to `false`.
    - `owner`: Specify which voices to return. Choose from `system`, `me`, or `all`. Defaults to `all`.

    Returns a list of dictionaries containing details of each voice.
    """
    if owner not in ['all', 'system', 'me']:
      raise ValueError(f'Invalid owner: {owner}')
    if not isinstance(starred, bool):
      raise ValueError(f'Invalid starred: {starred}')
    self._lazy_init()
    url = f'{self._base_url}{_LIST_VOICES_ENDPOINT}?starred={starred}&owner={owner}'

    async with self._session.get(url, headers=self._build_headers()) as resp:
      await self._handle_response_errors(resp, 'Speech.list_voices')
      return await resp.json()

  async def voice_info(self, voice_id: str):
    """
    Returns details of a specific voice.

    Required parameters:
    - `voice_id`: The id of the voice to get info on. If you don't know the id, you can get it from `list_voices()`.

    Returns a dictionary containing details of the voice.
    """
    self._lazy_init()
    url = f'{self._base_url}{_VOICE_ENDPOINT}'.format(id=voice_id)

    async with self._session.get(url, headers=self._build_headers()) as resp:
      await self._handle_response_errors(resp, 'Speech.voice_info')
      return await resp.json()

  async def create_voice(self, name: str, enhance: bool, filenames: List[str], type: str = 'instant', gender: str = None, description: str = None):
    """
    Creates a new voice from a set of audio files. Returns the voice metadata object.

    Parameters:
    - `name`: The name of the voice.
    - `enhance`: For unclean audio with background noise, applies processing to attempt to improve quality. Not on by default as it can also degrade quality in some circumstances.
    - `filenames`: A list of filenames to use for the voice.

    Optional parameters:
    - `type`: The type of voice to create. Must be one of `instant` or `professional`. Defaults to `instant`.
    - `gender`: The gender of the voice, e.g. `male`, `female`, `nonbinary`. For categorization purposes. Defaults to `None`.
    - `description`: A description of the voice. Defaults to `None`.

    Returns the voice metadata object:
    - `id`: The id of the voice (`voice_id`).
    - `name`: The name of the voice.
    - `owner`: The owner of the voice.
    - `state`: The state of the voice, e.g. `ready`, `pending`, `broken`.
    - `gender`: The gender of the voice, e.g. `male`, `female`, `nonbinary`.
    - `type`: The type of voice, e.g. `instant`, `professional`.
    - `description`: A description of the voice.
    """
    if type not in ['instant', 'professional']:
      raise ValueError(f'[Speech.create_voice] Invalid type: {type}')
    if name is None:
      raise ValueError('[Speech.create_voice] Name must not be None.')
    if len(name) == 0:
      raise ValueError('[Speech.create_voice] Name must be non-empty.')
    if filenames is None:
      raise ValueError('[Speech.create_voice] Filenames must not be None.')
    if len(filenames) == 0:
      raise ValueError('[Speech.create_voice] Filenames must be non-empty.')
    if enhance is None:
      raise ValueError('[Speech.create_voice] Enhance must not be None.')

    self._lazy_init()

    metadata = json.dumps({
        'name': name,
        'enhance': enhance,
        'type': type,
        'gender': gender,
        'description': description,
    })
    files = []
    try:
      with aiohttp.MultipartWriter() as mpwriter:
        mpwriter.append(metadata, {'Content-Type': 'application/json', 'Content-Disposition': 'form-data; name="metadata"'})
        for filename in filenames:
          f = open(filename, 'rb')
          files.append(f)
          part = mpwriter.append(f)
          part.set_content_disposition('form-data', name='file_field', filename=os.path.basename(filename))
      async with self._session.post(f'{self._base_url}{_CREATE_VOICE_ENDPOINT}', data=mpwriter, headers=self._build_headers()) as resp:
        await self._handle_response_errors(resp, 'Speech.create_voice')
        return await resp.json()
    finally:
      for file in files:
        file.close()

  async def update_voice(self, voice_id: str, **kwargs):
    """
    Updates metadata for a specific voice. A voice that is not owned by you can only have its `starred` field updated.
    Only provided fields will be changed.

    Required parameters:
    - `voice_id` (str): The id of the voice to update. If you don't know the id, you can get it from `list_voices()`.

    Optional parameters:
    - `name` (str): The name of the voice.
    - `starred` (bool): Whether the voice is starred by you
    - `gender` (str):  The gender of the voice, e.g. `male`, `female`, `nonbinary`. For categorization purposes.
    - `description` (str): A description of the voice.
    """
    self._lazy_init()
    url = f'{self._base_url}{_VOICE_ENDPOINT}'.format(id=voice_id)

    data = {
        'name': kwargs.get('name', None),
        'starred': kwargs.get('starred', None),
        'gender': kwargs.get('gender', None),
        'description': kwargs.get('description', None),
    }
    data = json.dumps({k: v for k, v in data.items() if v is not None})
    async with self._session.put(url, data=data, headers=self._build_headers(type='application/json')) as resp:
      await self._handle_response_errors(resp, 'Speech.update_voice')
      return await resp.json()

  async def unfreeze_voice(self, voice_id: str):
    """
    Unfreezes a professional voice clone owned by you.

    We are constantly improving and releasing our speech models. Voices that are not being used will not be
    upgraded automatically and will enter a frozen state.

    Do not worry though. If your voice is frozen, call this method to upgrade it to the latest model.

    Instant voices always use the latest model and are never frozen.
    """
    self._lazy_init()
    url = f'{self._base_url}{_VOICE_ENDPOINT}'.format(id=voice_id)
    data = {
        'unfreeze': True
    }
    data = json.dumps({k: v for k, v in data.items() if v is not None})
    async with self._session.put(url, data=data, headers=self._build_headers(type='application/json')) as resp:
      await self._handle_response_errors(resp, 'Speech.unfreeze_voice')
      return await resp.json()

  async def delete_voice(self, voice_id: str):
    """
    Deletes a voice and cancels any pending operations on it. The voice must be owned by you. Cannot be undone.

    Required parameters:
    - `voice_id` (str): The id of the voice to delete. If you don't know the id, you can get it from `list_voices()`.
    """
    self._lazy_init()
    url = f'{self._base_url}{_VOICE_ENDPOINT}'.format(id=voice_id)

    async with self._session.delete(url, headers=self._build_headers()) as resp:
      await self._handle_response_errors(resp, 'Speech.delete_voice')
      return await resp.json()

  async def synthesize(self, text: str, voice: str, **kwargs):
    """
    Synthesize speech from text.

    Parameters:
    - `text` (str): The text to synthesize.
    - `voice` (str): The voice id to use for synthesis.

    Optional parameters:
    - `seed` (int): The seed used to specify a different take. Defaults to random.
    - `format` (str): The audio format to use for synthesis. Defaults to `mp3`.
    - `sample_rate` (int): 8000, 16000, or 24000 – the desired output sample rate. Defaults to 24000 for all formats except `mulaw` which defaults to 8000.
    - `speed` (float): Floating point value between 0.25 (slow) and 2.0 (fast); Defaults to 1.0
    - `return_durations` (bool): If `True`, the response will include word durations detail. Defaults to `False`.
    - `return_seed` (bool): If `True`, the response will include the seed used for synthesis. Defaults to `False`.
    - `language` (str): The desired language of the synthesized speech. Two letter ISO 639-1 code. Defaults to `en`.
    - `length` (int): The desired target length of the output speech in seconds. Maximum 300.0 (5 minutes)

    Deprecated parameters:
    - `durations` (bool): If `True`, the response will include word durations detail. Defaults to `False`. Deprecated in favor of `return_durations`.

    Returns an object with the following keys:
    - `audio`: The binary audio file.
    - `durations`: The word durations detail. Only returned if `return_durations` is `True`.
    - `seed`: The seed used for synthesis. Only returned if `return_seed` is `True`.

    Each `durations` entry is a dictionary describing the duration of each word with the following keys:
    - `text`: the word itself
    - `start`: the time at which the word starts, in seconds
    - `duration`: the overall duration of the word, in seconds
    """
    assert text is not None, '[Speech.synthesize] `text` must not be None.'
    assert voice is not None, '[Speech.synthesize] `voice` must not be None.'
    assert len(text) > 0, '[Speech.synthesize] `text` must be non-empty.'
    assert len(voice) > 0, '[Speech.synthesize] `voice` must be non-empty.'

    self._lazy_init()
    url = f'{self._base_url}{_SPEECH_ENDPOINT}'

    form_data = aiohttp.FormData()
    form_data.add_field('text', text)
    form_data.add_field('voice', voice)
    if 'seed' in kwargs:
      form_data.add_field('seed', kwargs.get('seed'))
    form_data.add_field('format', kwargs.get('format', 'mp3'))
    form_data.add_field('speed', kwargs.get('speed', 1.0))
    length = kwargs.get('length', None)
    if length is not None:
      form_data.add_field('length', length)
    if 'sample_rate' in kwargs:
      form_data.add_field('sample_rate', kwargs.get('sample_rate'))
    if 'quality' in kwargs:
      form_data.add_field('quality', kwargs.get('quality'))
    return_durations = kwargs.get('durations', False)
    if 'return_durations' in kwargs:  # return_durations takes precedence over durations
      return_durations = kwargs['return_durations']
    if return_durations is True:
      form_data.add_field('return_durations', 'true')
    return_seed = kwargs.get('return_seed', False)
    if 'language' in kwargs:
      form_data.add_field('language', kwargs.get('language'))
    async with self._session.post(url, data=form_data, headers=self._build_headers()) as resp:
      await self._handle_response_errors(resp, 'Speech.synthesize')
      response_data = await resp.json()
      synthesis_result = {}
      synthesis_result['audio'] = base64.b64decode(response_data['audio'])
      if return_durations:
        synthesis_result['durations'] = response_data['durations']
      if return_seed:
        synthesis_result['seed'] = response_data['seed']
      return synthesis_result

  async def synthesize_streaming(self, voice: str, return_extras: bool = False, **kwargs):
    """
    Initiates a full-duplex streaming connection with the server that allows you to send text and receive audio in real-time.

    Parameters:
    - `format` (str): `mp3`, `raw`, or `ulaw` – the desired output format. Defaults to `mp3`.
    - `sample_rate` (int): 8000, 16000, or 24000 – the desired output sample rate. Defaults to 24000.
    - `voice` (str): The voice id to use for this connection.
    - `speed` (float): The speed to use for synthesis. Defaults to 1.0.
    - `return_extras` (bool): If `True`, the response will include word durations detail. Defaults to `False`.
    - `language` (str): The desired language of the synthesized speech. Two letter ISO 639-1 code. Defaults to `en`.

    Returns:
    - `StreamingSynthesisConnection`: The streaming connection object.
    """
    if not voice:
      raise ValueError('[Speech.synthesize_streaming] `voice` must not be None.')

    self._lazy_init()

    init_msg = {
        'X-API-Key': self._api_key,
        'voice': voice
    }
    if 'format' in kwargs:
      init_msg['format'] = kwargs['format']
    if 'sample_rate' in kwargs:
      init_msg['sample_rate'] = kwargs['sample_rate']
    if 'speed' in kwargs:
      init_msg['speed'] = kwargs['speed']
    if 'expressive' in kwargs:
      init_msg['expressive'] = kwargs['expressive']
    init_msg['send_extras'] = return_extras
    if 'language' in kwargs:
      init_msg['language'] = kwargs['language']
    ws = await self._session.ws_connect(f'{self._base_url}{_SYNTHESIZE_STREAMING_ENDPOINT}')
    await ws.send_str(json.dumps(init_msg))
    return StreamingSynthesisConnection(ws, return_extras)

  async def account_info(self):
    """
    Returns details about your account.
    """
    self._lazy_init()
    url = f'{self._base_url}{_ACCOUNT_ENDPOINT}'

    async with self._session.get(url, headers=self._build_headers()) as resp:
      await self._handle_response_errors(resp, 'Speech.account_info')
      return await resp.json()

  def _lazy_init(self):
    if self._session is None:
      self._session = aiohttp.ClientSession(connector=self._connector)

  def _build_headers(self, type: str = None):
    headers = {'X-API-Key': self._api_key}
    if type is not None:
      headers['Content-Type'] = type
    return headers

  async def _handle_response_errors(self, response, caller=None):
    if response.status < 400:
      return
    raise SpeechError(response.status, await response.json(), caller)



from argparse import ArgumentParser
import asyncio
from dotenv import load_dotenv
from lmnt.api import Speech
from openai import AsyncOpenAI
import os

load_dotenv()  # Don't forget to add your LMNT and OpenAI API keys to .env.

MODEL = 'gpt-3.5-turbo'
DEFAULT_PROMPT = 'Tell me an interesting fact about the universe.'
VOICE_ID = 'lily'


async def reader_task(conn):
  """Streams audio data from the server and writes it to `output.mp3`."""
  with open('output.mp3', 'wb') as f:
    async for msg in conn:
      f.write(msg['audio'])


async def writer_task(conn, prompt):
  """Streams text from ChatGPT to LMNT."""
  client = AsyncOpenAI(api_key=os.getenv('OPENAI_API_KEY'))
  response = await client.chat.completions.create(model=MODEL,
                                                  messages=[{'role': 'user', 'content': prompt}],
                                                  stream=True)

  async for chunk in response:
    if not chunk.choices[0] or not chunk.choices[0].delta or not chunk.choices[0].delta.content:
      continue
    content = chunk.choices[0].delta.content
    await conn.append_text(content)
    print(content, end='', flush=True)

  await conn.finish()


async def main(args):
  speech = Speech(os.getenv('LMNT_API_KEY'))
  conn = await speech.synthesize_streaming(VOICE_ID, return_extras=False, language=args.language)

  t1 = asyncio.create_task(reader_task(conn))
  t2 = asyncio.create_task(writer_task(conn, args.prompt))

  await t1
  await t2
  await speech.close()


if __name__ == '__main__':
  parser = ArgumentParser()
  parser.add_argument('prompt', default=DEFAULT_PROMPT, nargs='?')
  parser.add_argument('-l', '--language', required=False, default='en', help='Language code')
  asyncio.run(main(parser.parse_args()))



from unittest.mock import AsyncMock, patch, MagicMock
import os
import pytest
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', 'src', 'lmnt')))
from api import Speech, _LIST_VOICES_ENDPOINT, _VOICE_ENDPOINT  # noqa


def test_lazy_init():
  with patch('aiohttp.ClientSession', new=MagicMock()) as MockClientSession:
    api = Speech('test_key')
    api._lazy_init()
    MockClientSession.assert_called_once()
    assert isinstance(api._session, MagicMock)


@pytest.fixture
def mock_response():
  return {
      'voices': [
          {'name': 'Voice1', 'language': 'en-US'},
          {'name': 'Voice2', 'language': 'en-GB'},
      ]
  }


@pytest.fixture
async def api():
  with patch('aiohttp.ClientSession', new=MagicMock()) as MockClientSession:
    key = 'test_key'
    api = Speech(key)
    api._lazy_init()
    MockClientSession.assert_called_once()
    yield api


@pytest.mark.asyncio
async def test_list_voices_default_params(api, mock_response):
  with patch('aiohttp.ClientSession', new=MagicMock()):
    api._session.get.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
    api._session.get.return_value.__aenter__.return_value.status = 200
    result = await api.list_voices()
    api._session.get.assert_called_once_with(
        f'{api._base_url}{_LIST_VOICES_ENDPOINT}?starred=False&owner=all',
        headers=api._build_headers()
    )
    assert result == mock_response


@pytest.mark.asyncio
async def test_list_voices_with_params(api, mock_response):
  with patch('aiohttp.ClientSession', new=MagicMock()):
    api._session.get.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
    api._session.get.return_value.__aenter__.return_value.status = 200
    result = await api.list_voices(starred=True, owner='me')
    api._session.get.assert_called_once_with(
        f'{api._base_url}{_LIST_VOICES_ENDPOINT}?starred=True&owner=me',
        headers=api._build_headers()
    )
    assert result == mock_response


@pytest.mark.asyncio
async def test_list_voices_with_params_system(api, mock_response):
  with patch('aiohttp.ClientSession', new=MagicMock()):
    api._session.get.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
    api._session.get.return_value.__aenter__.return_value.status = 200
    result = await api.list_voices(starred=True, owner='system')
    api._session.get.assert_called_once_with(
        f'{api._base_url}{_LIST_VOICES_ENDPOINT}?starred=True&owner=system',
        headers=api._build_headers()
    )
    assert result == mock_response


@pytest.mark.asyncio
async def test_list_voices_invalid_owner(api):
  with pytest.raises(ValueError):
    await api.list_voices(owner='invalid')


@pytest.mark.asyncio
async def test_list_voices_starred(api):
  mock_response = [{'name': 'Voice1', 'starred': True}, {'name': 'Voice2', 'starred': True}]
  api._session.get.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.get.return_value.__aenter__.return_value.status = 200

  voices = await api.list_voices(starred=True)
  api._session.get.assert_called_once_with(
      f'{api._base_url}{_LIST_VOICES_ENDPOINT}?starred=True&owner=all',
      headers=api._build_headers()
  )
  assert voices == mock_response


@pytest.mark.asyncio
async def test_list_voices_invalid_star(api):
  with pytest.raises(ValueError):
    await api.list_voices(starred='invalid')


@pytest.mark.asyncio
async def test_voice_info(api):
  mock_response = {'id': 'Voice1', 'name': 'Voice1', 'language': 'en-US'}
  api._session.get.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.get.return_value.__aenter__.return_value.status = 200

  voice_info = await api.voice_info('Voice1')
  assert voice_info == mock_response


@pytest.mark.asyncio
async def test_voice_info_invalid_id(api):
  api._session.get.return_value.__aenter__.return_value.status = 400

  with pytest.raises(Exception):
    await api.voice_info('invalid')


@pytest.mark.asyncio
async def test_create_voice(api):
  mock_response = {'id': 'Voice1', 'name': 'Voice1', 'state': 'ready'}
  api._session.post.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.post.return_value.__aenter__.return_value.status = 200

  voice_info = await api.create_voice('Voice1', False, ['filename.wav', 'filename.wav'])
  assert voice_info == mock_response


@pytest.mark.asyncio
async def test_create_voice_invalid_type(api):
  with pytest.raises(ValueError):
    await api.create_voice('Voice1', False, ['filename.wav', 'filename.wav'], type='invalid')


@pytest.mark.asyncio
async def test_create_voice_no_name(api):
  with pytest.raises(ValueError):
    await api.create_voice(None, False, ['filename.wav', 'filename.wav'])


@pytest.mark.asyncio
async def test_create_voice_no_files(api):
  with pytest.raises(ValueError):
    await api.create_voice('Voice1', False, [])


@pytest.mark.asyncio
async def test_update_voice(api):
  mock_response = {'id': 'Voice1', 'name': 'UpdatedVoice', 'starred': True}
  api._session.put.return_value.__aenter__.return_value.json = AsyncMock(return_value=mock_response)
  api._session.put.return_value.__aenter__.return_value.status = 200

  voice_info = await api.update_voice('Voice1', name='UpdatedVoice', starred=True)
  assert voice_info == mock_response


@pytest.mark.asyncio
async def test_update_voice_invalid_id(api):
  api._session.put.return_value.__aenter__.return_value.status = 400

  with pytest.raises(Exception):
    await api.update_voice('invalid', name='UpdatedVoice', starred=True)


@pytest.mark.asyncio
async def test_delete_voice(api):
  voice_id = 'Voice1'
  api._session.delete.return_value.__aenter__.return_value.status = 200
  api._session.delete.return_value.__aenter__.return_value.json = AsyncMock(return_value={})

  response = await api.delete_voice(voice_id)
  api._session.delete.assert_called_once_with(
      f'{api._base_url}{_VOICE_ENDPOINT}'.format(id=voice_id),
      headers=api._build_headers()
  )
  assert response == {}


@pytest.mark.asyncio
async def test_delete_voice_invalid_id(api):
  api._session.delete.return_value.__aenter__.return_value.status = 400

  with pytest.raises(Exception):
    await api.delete_voice('invalid')



import argparse
import asyncio
import os
from dotenv import load_dotenv
from lmnt.api import Speech

load_dotenv()  # Don't forget to add your LMNT API key to .env.


async def main(args):
  async with Speech(os.getenv('LMNT_API_KEY')) as s:
    #  Get the list of available voices.
    voices = await s.list_voices()
    print(voices)

    # Get your account information.
    account = await s.account_info()
    print(account)

    # Synthesize text to speech.
    synthesize = await s.synthesize(text=args.text, voice=args.voice, language=args.language)
    with open('output.mp3', 'wb') as f:
      f.write(synthesize['audio'])
    print('Done.')

if __name__ == '__main__':
  parser = argparse.ArgumentParser(description='Synthesize text to speech using LMNT API')
  parser.add_argument('-t', '--text', required=False, default='This is a test of the LMNT API.', help='Text to synthesize')
  parser.add_argument('-v', '--voice', required=False, default='lily', help='Voice to use')
  parser.add_argument('-l', '--language', required=False, default='en', help='Language code')
  args = parser.parse_args()
  asyncio.run(main(args))



import asyncio
import pytest
import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..', 'src', 'lmnt')))
from api import Speech, StreamingSynthesisConnection  # noqa

# Set an API key in your environment to run these tests
X_API_KEY = os.environ['X_API_KEY']
_BASE_URL = 'https://api.lmnt.com'


@pytest.fixture
async def api():
  async with Speech(X_API_KEY, base_url=_BASE_URL) as speech:
    yield speech


@pytest.mark.asyncio
async def test_init(api: Speech):
  assert api is not None


async def reader_task_binary(connection):
  async for msg in connection:
    assert msg is not None
    assert 'audio' in msg
    assert 'durations' not in msg


@pytest.mark.asyncio
async def test_synthesize_streaming(api: Speech):
  voice = 'lily'
  connection = await api.synthesize_streaming(voice)
  assert connection is not None
  assert isinstance(connection, StreamingSynthesisConnection)
  reader = asyncio.create_task(reader_task_binary(connection))
  await connection.append_text('One, Hello, world!')
  await connection.append_text('Two, Hello, world!')
  await connection.append_text('Three, Hello, world!')
  await connection.finish()
  await reader


async def reader_task_str(connection):
  async for msg in connection:
    assert msg is not None
    assert 'audio' in msg
    assert 'durations' in msg


@pytest.mark.asyncio
async def test_synthesize_streaming_return_extras(api: Speech):
  voice = 'lily'
  connection = await api.synthesize_streaming(voice, return_extras=True)
  assert connection is not None
  assert isinstance(connection, StreamingSynthesisConnection)
  reader = asyncio.create_task(reader_task_str(connection))
  await connection.append_text('One, Hello, world!')
  await connection.append_text('Two, Hello, world!')
  await connection.append_text('Three, Hello, world!')
  await connection.finish()
  await reader


@pytest.mark.asyncio
async def test_durations(api: Speech):
  voice = 'lily'
  text = 'Example Text'
  result = await api.synthesize(text=text, voice=voice, return_durations=True)
  assert result is not None
  assert 'durations' in result
  assert 'phonemes' not in result
  assert 'audio' in result
  assert len(result['durations']) > 0
  assert len(result['audio']) > 0
  assert isinstance(result['audio'], bytes)


@pytest.mark.asyncio
async def test_synthesize_with_invalid_voice(api: Speech):
  voice = 'invalid_voice'
  text = 'Example Text'
  with pytest.raises(Exception):
    await api.synthesize(text=text, voice=voice)


@pytest.mark.asyncio
async def test_synthesize_with_empty_text(api: Speech):
  voice = 'lily'
  text = ''
  with pytest.raises(Exception):
    await api.synthesize(text=text, voice=voice)


@pytest.mark.asyncio
async def test_synthesize(api: Speech):
  voice = 'lily'
  text = 'Example Text'
  result = await api.synthesize(text=text, voice=voice)
  assert result is not None
  assert 'audio' in result
  assert 'durations' not in result
  assert 'seed' not in result
  assert len(result['audio']) > 0
  assert isinstance(result['audio'], bytes)


@pytest.mark.asyncio
async def test_synthesize__non_en_language(api: Speech):
  voice = 'lily'
  text = 'Example Text'
  language = 'pt'
  result = await api.synthesize(text=text, voice=voice, language=language)
  assert result is not None
  assert 'audio' in result
  assert 'durations' not in result
  assert 'seed' not in result
  assert len(result['audio']) > 0
  assert isinstance(result['audio'], bytes)


@pytest.mark.asyncio
async def test_synthesize_with_empty_voice(api: Speech):
  voice = ''
  text = 'Example Text'
  with pytest.raises(Exception):
    await api.synthesize(text=text, voice=voice)


@pytest.mark.asyncio
async def test_list_voices(api: Speech):
  voices = await api.list_voices()
  assert isinstance(voices, list)
  assert len(voices) > 0


@pytest.mark.asyncio
async def test_list_voices_starred(api: Speech):
  voices = await api.list_voices(starred=True)
  assert isinstance(voices, list)


@pytest.mark.asyncio
async def test_list_voices_owned(api: Speech):
  voices = await api.list_voices(owner='me')
  assert isinstance(voices, list)


@pytest.mark.asyncio
async def test_list_voices_owned_system(api: Speech):
  voices = await api.list_voices(owner='system')
  assert isinstance(voices, list)


@pytest.mark.asyncio
async def test_get_voice(api: Speech):
  voice = await api.voice_info('lily')
  assert isinstance(voice, dict)
  assert voice['name'] == 'Lily'


@pytest.mark.asyncio
async def test_get_voice_with_invalid_voice(api: Speech):
  with pytest.raises(Exception):
    await api.voice_info('invalid_voice')


@pytest.mark.asyncio
async def test_update_voice_star(api: Speech):
  await api.update_voice('lily', starred=True)
  voice = await api.voice_info('lily')
  assert voice.get('starred') is True
  await api.update_voice('lily', starred=False)
  voice = await api.voice_info('lily')
  assert voice.get('starred') is False


@pytest.mark.asyncio
async def test_create_voice_basic_no_filenames(api: Speech):
  with pytest.raises(Exception):
    await api.create_voice(name='test_voice', enhance=True)


@pytest.mark.asyncio
async def test_create_voice_basic_empty_filenames(api: Speech):
  with pytest.raises(Exception):
    await api.create_voice(name='test_voice', enhance=True, filenames=[])


@pytest.mark.asyncio
async def test_create_voice_basic_no_enhance(api: Speech):
  with pytest.raises(Exception):
    await api.create_voice(name='test_voice', filenames=['filename.wav'])


@pytest.mark.asyncio
async def test_create_voice_basic_no_name(api: Speech):
  with pytest.raises(Exception):
    await api.create_voice(enhance=True, filenames=['filename.wav'])


@pytest.mark.asyncio
async def test_create_voice_basic_empty_name(api: Speech):
  with pytest.raises(Exception):
    await api.create_voice(name='', enhance=True, filenames=['filename.wav'])


@pytest.mark.asyncio
async def test_create_voice_basic_invalid_filenames(api: Speech):
  with pytest.raises(Exception):
    await api.create_voice(name='test_voice', enhance=True, filenames=['invalid_filename.wav'])


@pytest.mark.asyncio
async def test_create_voice_advanced(api: Speech):
  voice = await api.create_voice(name='integration_test_voice', enhance=True, filenames=['filename.wav'], description='test description', gender='male', type='instant')
  voice_id = voice['id']
  stored_voice = await api.voice_info(voice_id)
  await api.delete_voice(voice_id)
  assert voice['name'] == 'integration_test_voice'
  assert voice['owner'] == 'me'
  assert voice['state'] == 'ready'
  assert voice['type'] == 'instant'
  voice.pop('state')  # stored state is not set to ready immediately after creation
  stored_voice.pop('state')
  assert voice == stored_voice
  with pytest.raises(Exception):
    await api.voice_info(voice_id)


@pytest.mark.asyncio
async def test_update_owned_voice(api: Speech):
  original_voice = await api.create_voice(name='test_voice_update', enhance=True, filenames=['filename.wav'], description='test description', gender='male', type='instant')
  voice_id = original_voice['id']
  original_stored_voice = await api.voice_info(voice_id)
  updated_voice = await api.update_voice(voice_id, description='my new description', gender='female', name='integration_new_name', starred=False)
  updated_stored_voice = await api.voice_info(voice_id)
  await api.delete_voice(voice_id)
  updated_voice = updated_voice['voice']

  assert original_voice['name'] == 'test_voice_update'
  assert original_voice['owner'] == 'me'
  assert original_voice['state'] == 'ready'
  assert original_voice['type'] == 'instant'
  assert original_voice['description'] == 'test description'
  assert original_voice['gender'] == 'male'
  original_voice.pop('state')  # stored state is not set to ready immediately after creation
  original_stored_voice.pop('state')
  assert original_voice == original_stored_voice

  assert updated_voice['name'] == 'integration_new_name'
  assert updated_voice['owner'] == 'me'
  assert updated_voice['type'] == 'instant'
  assert updated_voice['description'] == 'my new description'
  assert updated_voice['gender'] == 'female'
  updated_voice.pop('state')  # stored state is not set to ready immediately after creation
  updated_stored_voice.pop('state')
  assert updated_voice == updated_stored_voice
  with pytest.raises(Exception):
    await api.voice_info(voice_id)


@pytest.mark.asyncio
async def test_get_account_info(api):
  response = await api.account_info()
  assert isinstance(response, dict)



from setuptools import find_namespace_packages, setup


DESCRIPTION = 'Python client library for the LMNT API'
AUTHOR = 'LMNT, Inc.'
AUTHOR_EMAIL = 'feedback@lmnt.com'
URL = 'https://github.com/lmnt-com/lmnt-python'
LICENSE = 'Apache 2.0'
KEYWORDS = ['speech tts ai ml genai']
CLASSIFIERS = [
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: Apache Software License',
    'Operating System :: OS Independent',
]

setup(name='lmnt',
      description=DESCRIPTION,
      long_description=open('README.md', 'r').read(),
      long_description_content_type='text/markdown',
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      url=URL,
      license=LICENSE,
      keywords=KEYWORDS,
      packages=find_namespace_packages('src'),
      package_dir={'': 'src'},
      install_requires=[
          'aiohttp ~= 3.8'
      ],
      classifiers=CLASSIFIERS)



import argparse
import asyncio
import os
from dotenv import load_dotenv
from lmnt.api import Speech

load_dotenv()  # Don't forget to add your LMNT API key to .env.


async def main(args):
  async with Speech(os.getenv('LMNT_API_KEY')) as speech:
    # Get the list of available voices.
    voices = await speech.list_voices()
    print(voices)

    # Create a voice. Once completed, it will be available in your list of voices.
    create_voice = await speech.create_voice('Demo voice', True, args.input_files)
    print(create_voice)

if __name__ == '__main__':
  parser = argparse.ArgumentParser(description='Create a voice using LMNT API')
  parser.add_argument('-i', '--input_files', nargs='+', required=False, default=['sample-audio-input.mp3'], help='Input file path')
  args = parser.parse_args()
  asyncio.run(main(args))
